@@ -4,7 +4,7 @@
 // C++ source
 // This file is part of RGL.
 //
-// $Id: x11gui.cpp 766 2009-07-08 13:29:57Z murdoch $
+// $Id: x11gui.cpp 841 2012-01-06 19:11:34Z murdoch $
 
 // Uncomment for verbose output on stderr:
 // #define RGL_X11_DEBUG
@@ -26,6 +26,7 @@ public:
   X11WindowImpl(Window* in_window
   , X11GUIFactory* in_factory
   , ::Window in_xwindow
+  , XVisualInfo* invisualinfo
   );
   virtual ~X11WindowImpl();
   void setTitle(const char* title);
@@ -57,14 +58,16 @@ private:
   ::Window       xwindow;
   ::GLXContext   glxctx;
   friend class X11GUIFactory;
+  XVisualInfo* xvisualinfo;
 };
 // ---------------------------------------------------------------------------
 // X11WindowImpl Implementation
 // ---------------------------------------------------------------------------
-X11WindowImpl::X11WindowImpl(Window* w, X11GUIFactory* f, ::Window in_xwindow)
+X11WindowImpl::X11WindowImpl(Window* w, X11GUIFactory* f, ::Window in_xwindow, XVisualInfo* invisualinfo)
 : WindowImpl(w)
 , factory(f)
 , xwindow(in_xwindow)
+, xvisualinfo(invisualinfo)
 {
   on_init();
 }
@@ -73,6 +76,11 @@ X11WindowImpl::~X11WindowImpl()
 {
   if (xwindow != 0)
     destroy();
+  // free XVisualInfo structure
+  if (xvisualinfo) {
+    XFree(xvisualinfo);
+    xvisualinfo = 0;
+  }    
 }
 // ---------------------------------------------------------------------------
 void X11WindowImpl::setTitle(const char* title)
@@ -101,6 +109,8 @@ void X11WindowImpl::getWindowRect(int *left, int *top, int *right, int *bottom)
   ::Window root, child;
   int x, y;
   unsigned int width, height, border_width, depth;
+  factory->processEvents();
+  factory->flushX();
   XGetGeometry(factory->xdisplay, xwindow, &root, &x, &y, &width, &height, &border_width, &depth);
   XTranslateCoordinates(factory->xdisplay, xwindow, root, x, y, left, top, &child);
   XTranslateCoordinates(factory->xdisplay, xwindow, root, x+width, y+height, right, bottom, &child);
@@ -305,7 +315,7 @@ void X11WindowImpl::processEvent(XEvent& ev)
 // ---------------------------------------------------------------------------
 void X11WindowImpl::initGL()
 {  
-  glxctx = glXCreateContext(factory->xdisplay, factory->xvisualinfo, NULL, True);
+  glxctx = glXCreateContext(factory->xdisplay, xvisualinfo, NULL, True);
   if (!glxctx)
     factory->throw_error("unable to create GLX Context"); 
 }
@@ -440,7 +450,6 @@ static int X11SaveErr(Display *dsp, XErrorEvent *event)
 
 X11GUIFactory::X11GUIFactory(const char* displayname)
 : xdisplay(0)
-, xvisualinfo(0)
 , xfont(0)
 {
   // Open one display connection for all RGL X11 devices
@@ -468,24 +477,6 @@ X11GUIFactory::X11GUIFactory(const char* displayname)
     throw_error("GLX extension missing on server"); return;
   }
   
-  // Choose GLX visual
-  
-  static int attribList[] =
-  {
-    GLX_RGBA,
-    GLX_DOUBLEBUFFER,
-    GLX_RED_SIZE, 1,
-    GLX_GREEN_SIZE, 1,
-    GLX_BLUE_SIZE, 1,
-    GLX_ALPHA_SIZE, 0,
-    GLX_DEPTH_SIZE, 1,
-    None
-  };
-
-  xvisualinfo = glXChooseVisual( xdisplay, DefaultScreen(xdisplay), attribList );
-  if (xvisualinfo == 0) {
-    throw_error("no suitable visual available"); return;
-  }
 }
 // ---------------------------------------------------------------------------
 X11GUIFactory::~X11GUIFactory()
@@ -501,12 +492,6 @@ void X11GUIFactory::disconnect()
     processEvents();
   }
 
-  // free XVisualInfo structure
-  if (xvisualinfo) {
-    XFree(xvisualinfo);
-    xvisualinfo = 0;
-  }
-
   // free xfont
   if (xfont) {
     XUnloadFont(xdisplay, xfont->fid);
@@ -557,6 +542,50 @@ void X11GUIFactory::processEvents()
 WindowImpl* X11GUIFactory::createWindowImpl(Window* window)
 {
   X11WindowImpl* impl = NULL;
+  XVisualInfo* xvisualinfo;
+
+  // Choose GLX visual
+  
+  static int attribList[] =
+  {
+    GLX_RGBA,
+    GLX_DOUBLEBUFFER,
+    GLX_RED_SIZE, 1,
+    GLX_GREEN_SIZE, 1,
+    GLX_BLUE_SIZE, 1,
+    GLX_ALPHA_SIZE, 0,
+    GLX_DEPTH_SIZE, 1,
+    None, None, // Space for optional AA settings
+    None, None,
+    None
+  };
+
+#ifdef GLX_SAMPLE_BUFFERS
+  // Setup antialiasing based on "rgl.antialias" option
+  int aa;
+  SEXP rgl_aa = GetOption(install("rgl.antialias"),R_BaseEnv);
+  if (isNull(rgl_aa)) aa = RGL_ANTIALIAS;
+  else aa = asInteger(rgl_aa);
+  
+  if(aa > 0) {
+    attribList[12] = GLX_SAMPLE_BUFFERS;
+    attribList[13] = 1;
+    attribList[14] = GLX_SAMPLES;
+    attribList[15] = aa;
+  }
+#endif
+
+  xvisualinfo = glXChooseVisual( xdisplay, DefaultScreen(xdisplay), attribList );
+#ifdef GLX_SAMPLE_BUFFERS
+  // Try to set up visual without MSAA if it failed
+  if (xvisualinfo == 0 && aa > 0) {
+    attribList[12] = None;
+    xvisualinfo = glXChooseVisual( xdisplay, DefaultScreen(xdisplay), attribList );
+  }
+#endif
+  if (xvisualinfo == 0) {
+    throw_error("no suitable visual available"); 
+  }
     
   // create X11 window
 
@@ -622,7 +651,7 @@ WindowImpl* X11GUIFactory::createWindowImpl(Window* window)
 
   // create window implementation instance
   
-  impl = new X11WindowImpl(window, this, xwindow);
+  impl = new X11WindowImpl(window, this, xwindow, xvisualinfo);
 
   // register instance
   
